﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace QuakeCharsetUtility
{
    public partial class CharacterMapView : PictureBox
    {
        // a delegate is a method pointer basically. 
        public delegate void CharacterClickHandler(object sender, CharacterEventArgs e);

        // An event is .net's nice way of providing event functionality to user input and such. This one will be raised when a character is selected.
        public event CharacterClickHandler CharacterClick;

        private const int CHAR_COLUMN_COUNT = 16;
        private const int CHAR_ROW_COUNT = 16;

        public CharacterMapView()
        {
            InitializeComponent();

            // Load Default charset as Image
            this.Image = Properties.Resources.default_charset;
        }

        // Call this from the main form to set a different Charset if desired
        public void SetCharset(Image pCharsetImage)
        {
            this.Image = pCharsetImage;
        }

        // This event will be raised when the user's mouse button comes up
        protected override void OnMouseUp(MouseEventArgs e)
        {
            OnCharacterClick(GetCharacterFromCoords(e.Location));

            base.OnMouseUp(e);
        }

        private byte GetCharacterFromCoords(Point pLocationOfClick)
        {
            // Convert from Mouse X/Y to 16x16 grid row/col
            //   
            // Since we're dealing with ratios - where we're trying to figure out n:    n/16 = x/width
            // Multiply 16 by x, then device by width to figure out n. Same thing for y and height.

                    //Cast to int..     Cast to Double so Math.Floor knows what to expect... 
            int column = (int)Math.Floor((double)(pLocationOfClick.X * CHAR_COLUMN_COUNT) / this.Width);
            int row = (int)Math.Floor((double)(pLocationOfClick.Y * CHAR_ROW_COUNT) / this.Height);

            // Now convert to byte
            byte columnByte = (byte)column;
            byte rowByte = (byte)row;

            // return combined value, with column shifted by 4. 
            return (byte)(rowByte << 4 | columnByte);
        }

        private void OnCharacterClick(byte pCharacterByte)
        {
            // If this is not null, meaning an outsider has hooked into the event, then raise it.
            if (CharacterClick != null)
                CharacterClick(this, new CharacterEventArgs(pCharacterByte));
        }

        // boiler plate event args class..
        public class CharacterEventArgs : EventArgs
        {
            public byte CharacterByte { get; private set;}

            public CharacterEventArgs(byte pCharacterByte)
            {
                CharacterByte = pCharacterByte;
            }
        }
    }
}
